package protect.gift_card_guard;

import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CursorAdapter;
import android.widget.TextView;

class GiftCardCursorAdapter extends CursorAdapter
{
    public GiftCardCursorAdapter(Context context, Cursor cursor)
    {
        super(context, cursor, 0);
    }

    // The newView method is used to inflate a new view and return it,
    // you don't bind any data to the view at this point.
    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent)
    {
        return LayoutInflater.from(context).inflate(R.layout.giftcard_layout, parent, false);
    }

    // The bindView method is used to bind all data to a given view
    // such as setting the text on a TextView.
    @Override
    public void bindView(View view, Context context, Cursor cursor)
    {
        // Find fields to populate in inflated template
        TextView storeField = (TextView) view.findViewById(R.id.store);
        TextView currencyField = (TextView) view.findViewById(R.id.currency);
        TextView valueField = (TextView) view.findViewById(R.id.value);
        TextView cardIdField = (TextView) view.findViewById(R.id.cardId);

        // Extract properties from cursor
        GiftCard giftCard = GiftCard.toGiftCard(cursor);

        // Populate fields with extracted properties
        storeField.setText(giftCard.store);
        valueField.setText(giftCard.value);

        String defaultCurrency = view.getResources().getString(R.string.pref_default_currency);
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(view.getContext());
        String currency = preferences.getString("currency", defaultCurrency);
        currencyField.setText(currency);

        String cardIdFormat = view.getResources().getString(R.string.cardIdFormat);
        String cardIdLabel = view.getResources().getString(R.string.cardId);
        String cardIdText = String.format(cardIdFormat, cardIdLabel, giftCard.cardId);
        cardIdField.setText(cardIdText);
    }
}
